local psys;
local enemies;



local player = {}

function player:init()

    self.x = GOX
    self.y = GOY*1.4
    self.w = 92
    self.h = 92
    self.r = 0

    self.hw = self.w*.5
    self.hh = self.h*.5

    self.bullets = {}

end

function player:reset()
    self.x = nil
    self.y = nil
    self.w = nil
    self.h = nil
    self.r = nil
    self.hw = nil
    self.hh = nil
    self.bullets = nil
end

local turn = 1
local shoot_t = 0

function player:update(buttons, dt)

    turn = self.r < -.61 and 1 or self.r > .61 and -1 or turn

    for k, btn in pairs(buttons) do

        if btn.touchID then

            if k == "left" then
                self.x = math.max(self.x - 600 * dt, self.hw)

            elseif k == "right" then
                self.x = math.min(self.x + 600 * dt, GW-self.hw)
            end

            if k == "reset" then
                self.r = self.r + (2*turn) * dt
            end

            if k == "jump" then

                if shoot_t <= 0 then

                    shoot_t = .1
                    local r = self.r-1.5708

                    local c = math.cos(self.r)
                    local s = math.sin(self.r)

                    local hh = self.hh
                    local x = hh * s + self.x
                    local y = -hh * c + self.y

                    self.bullets[#self.bullets+1] = {
                        x = x, y = y,
                        vx = math.cos(r),
                        vy = math.sin(r)
                    }

                else
                    shoot_t = shoot_t - dt
                end

            end

        end

    end

    for i = #self.bullets, 1, -1 do

        local b = self.bullets[i]
        b.x = b.x + b.vx * 800 * dt
        b.y = b.y + b.vy * 800 * dt

        if b.x < 0 or b.x > GW or b.y < 0 then
            table.remove(self.bullets, i)
            goto continue
        end

        for j, e in ipairs(enemies) do

            local sradii = 16 + 4
            local dx = e.x - b.x
            local dy = e.y - b.y

            if dx*dx + dy*dy <= sradii*sradii then
                psys:setPosition(e.x, e.y)
                psys:emit(math.random(6,12))
                table.remove(self.bullets, i)
                table.remove(enemies, j)
                break
            end

        end

        ::continue::

    end

end

function player:draw()

    -- Draw bullets --

    for _, b in ipairs(self.bullets) do

        lg.setLineWidth(4)
        lg.setColor(1,1,1)
        lg.circle("line", b.x, b.y, 4)

        lg.setLineWidth(1)
        lg.setColor(0,0,0)
        lg.circle("fill", b.x, b.y, 4)

    end

    -- Draw spaceship --

    local x, y = self.x, self.y
    local w, h = self.w, self.h
    local r = self.r

    local hw = w * 0.5
    local hh = h * 0.5

    local c = math.cos(r)
    local s = math.sin(r)

    local x1, y1 = -hw, hh
    local x2, y2 = 0, -hh
    local x3, y3 = hw, hh

    x1, y1 = x1 * c - y1 * s + x, x1 * s + y1 * c + y
    x2, y2 = x2 * c - y2 * s + x, x2 * s + y2 * c + y
    x3, y3 = x3 * c - y3 * s + x, x3 * s + y3 * c + y

    lg.setLineWidth(6)
    lg.setColor(1,1,1)
    lg.line(x,y, x1,y1, x2,y2, x3,y3, x,y)
    lg.setLineWidth(1)

    lg.setColor(0,0,0)
    lg.polygon("fill", x,y, x1,y1, x2,y2, x3,y3, x,y)
    lg.setColor(1,1,1)

end



enemies = {}

function enemies:init()
    self.spawn_timer = 0
    self:reset()
end

function enemies:reset()
    if #self>0 then
        for i = 1, #self do
            self[i] = nil
        end
    end
end

function enemies:update(dt)

    self.spawn_timer = self.spawn_timer + dt

    if self.spawn_timer > .75 then

        self.spawn_timer = 0

        self[#self+1] = {
            x = math.random(0,GW), y = -16;
            vx = math.random(-10,10)/10, vy = 1;
        }

    end

    for i = #self, 1, -1 do

        local e = self[i]

        e.x = e.x + e.vx * 600 * dt
        e.y = e.y + e.vy * 600 * dt

        if e.x < -16 or e.x > GW+16 or e.y > GH+16 then
            table.remove(self, i)
            goto continue
        end

        local sradii = 16 + 32
        local dx = e.x - player.x
        local dy = e.y - player.y

        if dx*dx + dy*dy <= sradii*sradii then
            psys:setPosition(e.x, e.y)
            psys:emit(math.random(6,12))
            table.remove(self, i)
        end

        ::continue::

    end

end

function enemies:draw()

    for _, e in ipairs(self) do

        lg.setLineWidth(6)
        lg.setColor(1,1,1)
        lg.circle("line", e.x, e.y, 16)

        lg.setLineWidth(1)
        lg.setColor(0,0,0)
        lg.circle("fill", e.x, e.y, 16)

    end

    lg.setColor(1,1,1)

end



local DemoTouch = {}

function DemoTouch:reset()
    enemies:reset()
    player:reset()
    psys:release()
end

function DemoTouch:init()

    psys = lg.newParticleSystem(
        IMG_Particle, 32
    )

    psys:setLinearAcceleration(
        -400, -400, 400, 400
    )

    psys:setParticleLifetime(.5,1)

    psys:setColors(
        1,1,1,1, 1,1,1,0
    )

    enemies:init()
    player:init()

end

function DemoTouch:update(buttons, dt)
    player:update(buttons, dt)
    enemies:update(dt)
    psys:update(dt)
end

function DemoTouch:draw()
    enemies:draw()
    player:draw()
    lg.draw(psys)
end

return DemoTouch
